<?php

function cluster_priority_for_split( &$a, &$b )
{
  $aa = bounds_lat_range($a['bounds']) * bounds_lon_range($a['bounds']);
  $ab = bounds_lat_range($b['bounds']) * bounds_lon_range($b['bounds']);
  return -$aa+$ab;
}

class ClusterMaker
{
  var $_image_map;
  var $_bounds;
  
  var $debug_str;
  
  function ClusterMaker(&$images)
  {
    $this->_image_map = array();
    $this->_bounds = array();
    foreach($images as $img)
    {
      $ll = array( 'lat'=>$img['lat'], 'lon'=>$img['lon'] );
      $this->_image_map[ $img['id'] ] = $ll;
      $this->_bounds = bounds_add($this->_bounds, $ll['lat'], $ll['lon'] );
    }
    $this->debug_str = '';
  }

  function make_clusters($maxLatPrecision, $maxLonPrecision, $maxNbMarkers)
  {
    $start = get_moment();
    $total_iterations = 0;
    $total_generations = 0;
    
    $result = array();
    $pending_split = array(
        array(
          'bounds' => $this->_bounds,
          'items' => array_keys( $this->_image_map ),
        )
      );

    while ( count($pending_split) )
    {
      $total_generations++;
      $next_level_to_split = array();
      while ( count($pending_split) )
      {
        $current = array_shift( $pending_split );
        $splitted = $this->_split_cluster($current, $maxLatPrecision, $maxLonPrecision );
        if ( count($splitted)>1 )
        {
/*          echo('splitted: ' . var_export($current['bounds'],true). ' in '. count($splitted)."\n" );
          foreach( $splitted as $k => $split )
            echo("sub $k: " . var_export($split['bounds'],true)."\n" );*/
          $total_iterations += count($current['items']);
          $next_level_to_split = array_merge($next_level_to_split, $splitted );
          if ( count($result)+count($pending_split)+count($next_level_to_split) >= $maxNbMarkers )
          {
            $result = array_merge($result, $pending_split, $next_level_to_split );
            $pending_split = $next_level_to_split = array();
            break;
          }
        }
        else
          array_push( $result, $current );
      }
      $pending_split = $next_level_to_split;
      usort( $pending_split, 'cluster_priority_for_split' );
    }
    
    $this->debug_str = get_elapsed_time($start, get_moment()) .' gen:'.$total_generations.' alg:'.count($this->_image_map).'+'.$total_iterations;
    return $result;
  }
  
  function _split_cluster($cluster, $maxLatPrecision, $maxLonPrecision)
  {
    $latRange = bounds_lat_range($cluster['bounds']);
    $lonRange = bounds_lon_range($cluster['bounds']);
    
    $lat_nb_tiles = max( 1, $latRange/$maxLatPrecision );
    $lon_nb_tiles = max( 1, $lonRange/$maxLonPrecision );
    
    if ($lat_nb_tiles==1 and $lon_nb_tiles==1)
      return array();
    
    $ll_tile_factor = $lat_nb_tiles/$lon_nb_tiles;
    
    if ($ll_tile_factor > 3)
    { // 3x1 split
      $lat_step = $latRange/3;
      $lon_step = $lonRange;
    }
    elseif ($ll_tile_factor < 1/3)
    { // 1x3 split
      $lat_step = $latRange;
      $lon_step = $lonRange/3;
    }
    elseif ($ll_tile_factor > 2)
    { // 3x2 split
      $lat_step = $latRange/3;
      $lon_step = max( $lonRange/2, $maxLonPrecision );
    }
    elseif ($ll_tile_factor < 1/2)
    { // 2x3 split
      $lat_step = max( $latRange/2, $maxLatPrecision );
      $lon_step = $lonRange/3;
    }
    else
    { // 3x3 split
      $lat_step = max( $latRange/3, $maxLatPrecision );
      $lon_step = max( $lonRange/3, $maxLonPrecision );
    }

    $lat_step += 1e-7;
    $lon_step += 1e-7;
    //echo ( "$lat_step $latRange x $lon_step $lonRange tiles $lat_nb_tiles x $lon_nb_tiles\n" );

    $lon_nb_tiles = ceil( $lonRange / $lon_step );

    $clusters = array();
    foreach ( $cluster['items'] as $id )
    {
      $lon = $this->_image_map[$id]['lon'];
      $lat = $this->_image_map[$id]['lat'];
      
      $idx_lon = floor ( ( $lon - $cluster['bounds']['w'] ) / $lon_step );
      $idx_lat = floor ( ( $lat - $cluster['bounds']['s'] ) / $lat_step );
      
      $idx = $lon_nb_tiles * $idx_lat + $idx_lon;
      
      if ( !isset($clusters[$idx]) )
      {
        $outer_bounds = array(
          's' => $cluster['bounds']['s'] + $idx_lat*$lat_step,
          'n' => $cluster['bounds']['s'] + $idx_lat*($lat_step + 1),
          'w' => $cluster['bounds']['w'] + $idx_lon*$lon_step,
          'e' => $cluster['bounds']['w'] + $idx_lon*($lon_step + 1),
          );
        $clusters[$idx] = array(
          'outer_bounds'  => $outer_bounds,
          'bounds'  => array(),
          'items'   => array(),
          );
      }
      array_push( $clusters[$idx]['items'], $id );
      $clusters[$idx]['bounds'] = bounds_add( $clusters[$idx]['bounds'] , $lat, $lon );
    }
    return $clusters;
  }
}

?>